#version 400

// TODO: 
// All the GBuffer stuff should be in a separate file (cleaner, no duplication).
// LoadShader() should pass all sources of each type (e.g. .frag) to a ShaderObject.
// (And prepend "#version 400" as another source string).

layout(location = 0) out vec4 GBuffer0;
layout(location = 1) out vec4 GBuffer1;

void SetAlbedo(vec3 albedo)
{
	GBuffer0.rgb = albedo;
}

void SetSpecularStrength(float value)
{
	GBuffer0.a = value;
}

void SetNormal(vec3 normal)
{
	GBuffer1.xyz = normal * 0.5 + 0.5;
}

float TAG_SCENE = 0.0;
float TAG_SKYBOX = 1.0 / 255.0;

void SetTag(float tag)
{
	GBuffer1.a = tag;
}

// N.B.
// Wireframe rendering from here: http://cgg-journal.com/2008-2/06/index.html (Shader Based Wireframe Drawing)

const vec3 WireColor = vec3(0.1, 0.1, 0.1);
noperspective in vec3 DistanceFromEdge;
uniform int ShowWireframe = 0;

in vec2 TexCoords;
in vec3 Position;
in vec2 DepthPos;

uniform vec2 TexCoordOrigin;
uniform vec2 TexCoordScale;

uniform sampler2D HeightsTexture;
uniform sampler2D NormalsTexture;

uniform sampler2D AlphasTexture;
uniform sampler2D DetailGrassTexture, DetailMudTexture;

uniform float DetailTextureScale;

vec3 AddWireframe(vec3 color)
{
	float d = min(DistanceFromEdge[0], min(DistanceFromEdge[1], DistanceFromEdge[2]));
 	float I = exp2(-2 * d * d);
 	
	return mix(color, WireColor, I * ShowWireframe);
}

void main()
{
	vec2 texCoords = TexCoordOrigin + TexCoords * TexCoordScale;
	
	//float height = texture(HeightsTexture, texCoords).x;
	vec3 normal = texture(NormalsTexture, texCoords).rgb;
	
	float alpha = texture(AlphasTexture, texCoords).x;
	
	vec2 detailTexCoords = (texCoords / TexCoordScale) * DetailTextureScale;
	
	vec3 detailGrass = texture(DetailGrassTexture, detailTexCoords).rgb;
	vec3 detailMud = texture(DetailMudTexture, detailTexCoords).rgb;
	
	vec3 detailBlended = mix(detailMud, detailGrass, alpha);
	
	vec3 color = AddWireframe(detailBlended);
	
	SetAlbedo(color);
	SetSpecularStrength(0.2);
	SetNormal(normal);
	SetTag(TAG_SCENE);
}
