#version 400

// subdivide into quads
layout(vertices = 4) out;

// Uses the technique described here:
// https://developer.nvidia.com/content/dynamic-hardware-tessellation-basics
// Code to calculate tessellation levels is taken more or less directly from the relevant NVidia OpenGL sample found here:
// https://developer.nvidia.com/gameworksdownload#?tx=$gameworks,graphics_library$graphics_api,opengl_4.3

const float MinTessLevel = 1.0;
const float MaxTessLevel = 64.0;

uniform vec2 WindowSize;

uniform mat4 ModelViewMatrix;
uniform mat4 ProjectionMatrix;

uniform float EdgeSize;

float GetTessellation(vec2 s0, vec2 s1)
{
	float d = distance(s0, s1);
	return clamp(d / EdgeSize, MinTessLevel, MaxTessLevel);
}

vec2 ViewToScreen(vec4 p)
{
	vec4 projected = ProjectionMatrix * p;
	
	vec2 ndc = (projected.xy / projected.w) * 0.5 + 0.5;
	
	return ndc * WindowSize;
}

float level(vec3 v1, vec3 v2)
{
	float diameter = distance(v1, v2);
	
	vec3 center = (v1 + v2) / 2.0;
	
	vec4 view0 = ModelViewMatrix * vec4(center, 1.0);
	vec4 view1 = vec4(view0.x + diameter, view0.yzw);
	
	vec2 s0 = ViewToScreen(view0);
	vec2 s1 = ViewToScreen(view1);
	
	return GetTessellation(s0, s1);
}

void main()
{
	if (gl_InvocationID == 0)
	{
		vec3 v0 = gl_in[0].gl_Position.xyz;
		vec3 v1 = gl_in[1].gl_Position.xyz;
		vec3 v2 = gl_in[2].gl_Position.xyz;
		vec3 v3 = gl_in[3].gl_Position.xyz;
		
		float l0 = level(v0, v3);
		float l1 = level(v0, v1);
		float l2 = level(v1, v2);
		float l3 = level(v2, v3);
		
		gl_TessLevelInner[0] = mix(l0, l2, 0.5);
		gl_TessLevelInner[1] = mix(l1, l3, 0.5);
		gl_TessLevelOuter[0] = l0;
		gl_TessLevelOuter[1] = l1;
		gl_TessLevelOuter[2] = l2;
		gl_TessLevelOuter[3] = l3;
	}
	
	gl_out[gl_InvocationID].gl_Position = gl_in[gl_InvocationID].gl_Position;
}
