#version 400

layout(quads, fractional_odd_spacing, ccw) in;

uniform vec2 TexCoordOrigin;
uniform vec2 TexCoordScale;

uniform mat4 ModelViewProjectionMatrix;

uniform sampler2D HeightsTexture;

out vec2 VTexCoords;
out vec3 VPosition;
out vec2 VDepthPos;

void main()
{
	VTexCoords = gl_TessCoord.xy;
	
	vec2 a = mix(gl_in[0].gl_Position.xz, gl_in[1].gl_Position.xz, gl_TessCoord.x);
	vec2 b = mix(gl_in[3].gl_Position.xz, gl_in[2].gl_Position.xz, gl_TessCoord.x);
	vec2 position_xz = mix(a, b, gl_TessCoord.y);
	
	// Get the height again, because the vertex shader only looked up height at the patch corners.
	// The height map is more detailed than that!
	
	float height = texture(HeightsTexture, TexCoordOrigin + gl_TessCoord.xy * TexCoordScale).x;
	vec3 position = vec3(position_xz.x, height, position_xz.y);
	
	VPosition = position;
	
	vec4 projectedPosition = ModelViewProjectionMatrix * vec4(position, 1.0);
	
	gl_Position = projectedPosition;
	
	VDepthPos = projectedPosition.zw;
}
